// boxControl.cpp
//
//	This file defines a simple example of a REALbasic plug-in control.
//	It makes a box, with a black outline and diagonal slash, and filled
//	with a user-definable color.  It has a "MakeRed" method which, when
//	called, sets the color to red, and also invokes an event handler.
//
//----------------------------------------------------------------------

#include "rb_plugin.h"

#if TARGET_WIN32
	#include <windows.h>
	#include <windowsx.h>
	#include <ddraw.h>
	#include "WINGDI.H"
	
	#include "shlobj.h"

#endif

#if TARGET_WIN32
typedef HRESULT (WINAPI *DDCreate) ( GUID FAR *lpGUID, LPDIRECTDRAW FAR *lplpDD, IUnknown FAR *pUnkOuter );
typedef HRESULT (WINAPI *DDCreateX) ( GUID FAR *lpGUID,LPDIRECTDRAW7 FAR *lplpDD,REFIID iid,IUnknown FAR *pUnkOuter );
#endif

extern struct REALcontrol generalplugControl;

//dont need this
REALevent generalplugEvents[] = {
	{ "PlugError()" }
};

struct generalplugdata
{
	int fillColor;			// color in RB color format (xxRRGGBB)
	int	reserror;
	
#if TARGET_WIN32
	LPDIRECTDRAW	pDirectDraw;
	LPDIRECTDRAW7	pDirectDrawX;
	DDCreate		lpfnDirectDrawCreate;
	DDCreateX		lpfnDirectDrawCreateX;
	HMODULE			drawlib;

#else
	int				pad1;
	int				pad2;
	int				pad3;
	int				pad4;
	int				pad5;
#endif
};
/*
#pragma mark generalplugInit
#ifdef TARGET_WIN32
static void generalplugInit(REALcontrolInstance instance)//, REALgraphics context)
#else
static void generalplugInit(REALcontrolInstance instance)
#endif
*/
void generalgone(REALcontrolInstance instance)
{
ControlData(generalplugControl, instance, generalplugdata, data);
#if TARGET_WIN32

data->pDirectDraw->Release();
data->pDirectDraw=NULL;

#else
#endif
}

static void generalplugInit(REALcontrolInstance  instance)
{
	ControlData(generalplugControl, instance, generalplugdata, data);

#if TARGET_WIN32

	//ControlData(generalplugControl, instance, generalplugdata, data);

	data->pDirectDraw=NULL;
	data->pDirectDrawX=NULL;

	SetErrorMode(SEM_NOOPENFILEERRORBOX);
	data->drawlib=LoadLibrary("Ddraw.dll");

	if (data->drawlib == NULL)
		goto exit_gracefully;

	///data->lpfnDirectDrawCreateX = (DDCreateX)GetProcAddress(data->drawlib,"DirectDrawCreateEx");
	///if (data->lpfnDirectDrawCreateX ==NULL)
		data->lpfnDirectDrawCreate = (DDCreate)GetProcAddress(data->drawlib,"DirectDrawCreate");
	if (data->lpfnDirectDrawCreate ==NULL)
		goto exit_gracefully;

	/////////if (data->lpfnDirectDrawCreateX !=NULL)
	{
	////if ( data->lpfnDirectDrawCreateX(NULL,&data->pDirectDrawX,IID_IDirectDraw7,NULL) != DD_OK )
		data->pDirectDrawX=NULL;
	}
	/////////else
	{
	if ( data->lpfnDirectDrawCreate(NULL,&data->pDirectDraw,NULL) != DD_OK )
		data->pDirectDraw=NULL;
	}
	
exit_gracefully:

return;

#endif

}

#pragma mark generalplugDraw
#ifdef TARGET_WIN32
static void generalplugDraw(REALcontrolInstance instance, REALgraphics context)
#else
static void generalplugDraw(REALcontrolInstance instance)
#endif
{

Rect r;
REALGetControlBounds(instance, &r);

	// Get our custom data into "data"
ControlData(generalplugControl, instance, generalplugdata, data);

	// Do the drawing (using native graphics toolbox)
#if TARGET_CARBON
	// Mac implementation:
RGBColor oldCol, fillCol, lineCol = {0,0,0};
GetForeColor(&oldCol);
fillCol.red =0;
fillCol.green =0;
fillCol.blue =0;
	
RGBForeColor(&fillCol);
FrameRect(&r);

MoveTo(r.left, r.top);
LineTo(r.right - 1, r.bottom - 1);
MoveTo(r.right, r.top);
LineTo(r.left - 1, r.bottom - 1);

RGBForeColor(&oldCol);
#else
#endif
}


//	{ (REALproc) generalplugsetresolution, REALnoImplementation, "SetRes (width as integer, height as integer, depth as integer)" },
//static void generalplugMakeRed(REALcontrolInstance instance)

#if TARGET_CARBON
DMDisplayModeListEntryRec mrec;
pascal void MyDMDisplayModeListIteratorCallback ( void *userData, DMListIndexType itemIndex, DMDisplayModeListEntryPtr displaymodeInfo)
{
mrec=(*displaymodeInfo);
}
#endif

#if TARGET_WIN32
typedef HRESULT (WINAPI *fold) (HWND hwndOwner, int nFolder, LPITEMIDLIST * ppidl);
typedef HRESULT (WINAPI *name) (LPCITEMIDLIST pidl, LPSTR pszPath);
#endif

static REALstring generalpluggetfolderpath(REALcontrolInstance instance, int f)
{
REALstring		s;

#if TARGET_WIN32
long			foldid;
LPITEMIDLIST	pidl = NULL;
LPSTR			szPath[MAX_PATH];
static			HMODULE			shelllib;
fold			addr;
name			namestr;
#endif

ControlData(generalplugControl, instance, generalplugdata, data);

#if TARGET_CARBON
s=REALBuildString("Error",5);
#else

addr=NULL;

SetErrorMode(SEM_NOOPENFILEERRORBOX);
shelllib=LoadLibrary("Shell32.dll");

if (shelllib == NULL)
	goto exit_gracefully;

addr =(fold)GetProcAddress(shelllib,"SHGetSpecialFolderLocation");

if (addr ==NULL)
	goto exit_gracefully;

namestr =(name)GetProcAddress(shelllib,"SHGetPathFromIDList");

if (namestr ==NULL)
	goto exit_gracefully;

foldid=f;
addr(NULL,foldid,&pidl);


namestr(pidl,(LPSTR)szPath);

s=REALBuildString((char*)szPath,strlen((const char*)szPath));

exit_gracefully:		
if(!pidl)
	free(pidl);

#endif

return s;
}

static void generalplugsetresolution(REALcontrolInstance instance, int w,int h,int d)
{

int	reterror=0;

// get our private data
ControlData(generalplugControl, instance, generalplugdata, data);

#if TARGET_CARBON
	OSErr	err;
	DMDisplayModeListIteratorUPP MyDMDisplayModeListIteratorUPP; 
	Handle	dispstate;
	DMListIndexType cnt;
	DMListType plist;
	DisplayIDType	id;
	Rect mr;
	GDHandle md;
	unsigned long dp;
	
	dp=d;
	md=DMGetFirstScreenDevice (true);

	err=DMBeginConfigureDisplays (&dispstate);
	
	MyDMDisplayModeListIteratorUPP = NewDMDisplayModeListIteratorUPP(&MyDMDisplayModeListIteratorCallback); 
 
 	SetRect(&mr,0,0,w,768);
 	reterror=-1;
 	
 	DMGetDisplayIDByGDevice (md,&id,true); 
	DMNewDisplayModeList (id,0,0,&cnt,&plist);
	for(int nn=0;nn<=cnt;nn++)
		{
		DMGetIndexedDisplayModeFromList (plist,nn,0,MyDMDisplayModeListIteratorUPP,nil);
		if((*mrec.displayModeResolutionInfo).csHorizontalPixels==mr.right)
			{
			err=DMSetDisplayMode (md,(*mrec.displayModeResolutionInfo).csDisplayModeID,&dp,NULL,dispstate);
			if (err!=noErr)
				reterror=-1;
			else
				reterror=1;
			}
		}
		
	err=DMEndConfigureDisplays (dispstate);

#else

	//ControlData(generalplugControl, instance, generalplugdata, data);
	///////if(data->pDirectDrawX==NULL)
REALwindow	realwind;
HWND		hMainWnd;
realwind=REALGetControlWindow(instance);
hMainWnd=REALGetWindowHandle(realwind);
//DDSCL_FULLSCREEN
reterror=1;

data->pDirectDraw->SetCooperativeLevel(hMainWnd,DDSCL_EXCLUSIVE);

if (data->pDirectDraw->SetDisplayMode(w,h,d)!=DD_OK)
	reterror=-1;

	//if (data->pDirectDrawX->SetCooperativeLevel((HWND)hMainWnd,DDSCL_EXCLUSIVE|DDSCL_FULLSCREEN)!=DD_OK)
	/////////if (data->pDirectDrawX->SetCooperativeLevel(nil,DDSCL_EXCLUSIVE|DDSCL_FULLSCREEN)!=DD_OK)
		//retcode=3;
	///////	if (data->pDirectDrawX->SetDisplayMode(w,h,d,0,DDSDM_STANDARDVGAMODE)!=DD_OK)
	//////	{
	//////	data->pDirectDrawX->Release();
	//////	data->pDirectDrawX=NULL;
		//retcode=2;
	/////	goto exit_gracefully;
	//////	}

#endif
	
data->reserror=reterror;

if(reterror!=1)
	{
	void (*fp)(REALcontrolInstance instance);
	fp = (void (*)(REALcontrolInstance instance)) REALGetEventInstance(instance, &generalplugEvents[0]);
	if (fp)
		fp(instance);
	}

return;	
	
}

REALproperty generalplugProperties[] = {
	{ "Data", "ResolutionError", "Integer", REALpropInvalidate, REALstandardGetter, REALstandardSetter, FieldOffset(generalplugdata, reserror) }

};


REALmethodDefinition generalplugMethods[] = {
	{ (REALproc) generalplugsetresolution, REALnoImplementation, "SetRes (width as integer, height as integer, depth as integer)" },
	{ (REALproc) generalpluggetfolderpath, REALnoImplementation, "GetFolderPath(f as Integer) As String" }
	
};

REALcontrolBehaviour generalplugBehaviour = {
	generalplugInit,
	generalgone,
	generalplugDraw,
	nil,
	nil,
	nil
};

REALcontrol generalplugControl = {
	kCurrentREALControlVersion,
	"GeneralPlug",
	sizeof(generalplugdata),
	REALinvisibleControl,					// for invisible controls like Timer, use: REALinvisibleControl
	128,
	129,
	32, 32,
	generalplugProperties,
	sizeof(generalplugProperties) / sizeof(REALproperty),
	generalplugMethods,
	sizeof(generalplugMethods) / sizeof(REALmethodDefinition),
	generalplugEvents,
	sizeof(generalplugEvents) / sizeof(REALevent),
	//nil,
	//sizeof(nil)/sizeof(REALevent),
	&generalplugBehaviour
};

void PluginEntry(void)
{
	REALRegisterControl(&generalplugControl);
}
